import os
import subprocess

try:
    GIT_HASH = subprocess.check_output(["git", "rev-parse", "--short", "HEAD"]).strip().decode("utf-8")
except Exception:
    GIT_HASH = "unknown"

G_FILE_PATH = os.path.join(os.path.dirname(__file__), "..", "graphs_3_7_c.g6")

SAT_SOLVER_NAME = "g4"

COOKIE_NAMES = [
    "oatmeal",
    "chocolate chip",
    "peanut butter",
    "sugar",
    "gingerbread",
    "shortbread",
    "snickerdoodle",
    "macadamia",
    "raisin",
    "butter",
    "almond",
    "hazelnut",
    "cinnamon",
    "lemon",
    "coconut",
    "coffee",
    "caramel",
    "pumpkin",
    "mint",
    "vanilla",
    "pistachio",
    "nutmeg",
    "pecan",
    "marzipan",
    "banana",
    "cranberry",
    "blueberry",
    "lime",
    "apricot",
    "walnut",
    "fig",
    "maple",
    "orange",
    "lavender",
    "cherry",
    "chai",
    "mocha",
    "sesame",
    "molasses",
    "matcha",
    "apple",
    "blackberry",
    "cardamom",
    "date",
    "espresso",
    "anise",
    "papaya",
    "praline",
    "pear",
    "rose",
    "toffee",
    "ube",
    "kiwi",
    "guava",
    "plum",
    "tapioca",
    "yam",
    "mango",
    "oreo",
    "figgy",
    "brandy",
    "rum",
    "thyme",
    "spice",
]


CHARACTERS = [
    "Quoll",
    "Skua",
    "Falcon",
    "Jacana",
    "Whimbrel",
    "Anole",
    "Coelacanth",
    "Bowerbird",
    "Mantis",
    "Neonate",
    "Heron",
    "Barnacle",
    "Puffbird",
    "Platypus",
    "Redstart",
    "Mola",
    "Penguin",
    "Cockatoo",
    "Fennec",
    "Anglerfish",
    "Hyrax",
    "Gecko",
    "Howler",
    "Katydid",
    "Monarch",
    "Cardinal",
    "Pangolin",
    "Irrawaddy",
    "Anemone",
    "Gibbon",
    "Magpie",
    "Pipefish",
    "Ayeaye",
    "Mandarin",
    "Jerboa",
    "Hoopoe",
    "Kingfisher",
    "Chough",
    "Lemming",
    "Batis",
    "Drill",
    "Bushbaby",
    "Nuthatch",
    "Quokka",
    "Iora",
    "Swiftlet",
    "Gerenuk",
    "Mink",
    "Ixodes",
    "Bobolink",
    "Dog",
    "Chipmunk",
    "Orca",
    "Galah",
    "Platybe",
    "Numbat",
    "Grackle",
    "Manakin",
    "Dory",
    "Quail",
    "Dikdik",
    "Kinkajou",
    "Squid",
    "Lacewing",
    "Tenrec",
    "Noddle",
    "Frigatebird",
    "Nautilus",
    "Olingo",
    "Baboon",
    "Pelican",
    "Cardinalfish",
    "Oystercatcher",
    "Finch",
    "Caddisfly",
    "Flycatcher",
    "Jacamar",
    "Aphid",
    "Crossbill",
    "Drongo",
    "Nehushtan",
    "Saki",
    "Lapwing",
    "Koala",
    "Ibis",
    "Nerodia",
    "Kangaroo",
    "Caecilian",
    "Lemur",
    "Butterfly",
    "Marten",
    "Longspur",
    "Bobcat",
    "Krill",
    "Peregrine",
    "Blenny",
    "Tapir",
    "Bilby",
    "Fairyshrimp",
    "Puffin",
    "Goeldi",
    "Bonobo",
    "Cuttlefish",
    "Narwhal",
    "Quicon",
    "Isopod",
    "Harrier",
    "Chinchilla",
    "Eland",
    "Tortoise",
    "Fox",
    "Flamingo",
    "Macaw",
    "Potoroo",
    "Starfish",
    "Stonefly",
    "Icterine",
    "Condor",
    "Markhor",
    "Inia",
    "Dobsonfly",
    "Bulbul",
    "Avocet",
    "Junco",
    "Sifaka",
    "Dovekie",
    "Ixia",
    "Barbet",
    "Kudu",
    "Oriole",
    "Budgie",
    "Damselfly",
    "Quetzal",
    "Antbird",
    "Ladybird",
    "Lyrebird",
    "Myna",
    "Quelea",
    "Meerkat",
    "Urchin",
    "Springhare",
    "Plover",
    "Fieldfare",
    "Qualis",
    "Moorhen",
    "Fluke",
    "Hoatzin",
    "Iulu",
    "Canary",
    "Lark",
    "Colobus",
    "Pika",
    "Kestrel",
    "Capybara",
    "Coati",
    "Dormouse",
    "Anteater",
    "Pipistrelle",
    "Quadrula",
    "Dunnart",
    "Leafy",
    "Firefly",
    "Quimera",
    "Manx",
    "Kiwi",
    "Osprey",
    "Dhole",
    "Ichneumon",
    "Panda",
    "Egret",
    "Jaguarundi",
    "SugarGlider",
    "Angelfish",
    "Sunbear",
    "Gopher",
    "Goby",
    "Jabiru",
    "Kinglet",
    "Wallaroo",
    "Owl",
    "Skylark",
    "Echidna",
    "Otter",
    "Dingo",
    "Peacock",
    "Terrapin",
    "Erythrocharax",
    "Beluga",
    "Nicator",
    "Loris",
    "Quahog",
    "Redpoll",
    "Grosbeak",
    "Capuchin",
    "Hamster",
    "Jay",
    "Tamarin",
    "Junonia",
    "Butterflyfish",
    "Jellyfish",
    "Bettong",
    "Bunting",
    "Nightjar",
    "Eagle",
    "Neurobasis",
    "Gannet",
    "Tigon",
    "Wagtail",
    "Jackdaw",
    "Peccary",
    "Stormpetrel",
    "Pipit",
    "Manul",
    "Hornbill",
    "Octopus",
    "Ferret",
    "Porpoise",
    "Albatross",
    "Chameleon",
    "Spatangoida",
    "Marmot",
    "Blobfish",
    "Fantail",
    "Erinaceus",
    "Weaver",
    "Caracal",
    "Roadrunner",
    "Titi",
    "Goldfish",
    "Mockingbird",
    "Seadragon",
    "Moth",
    "Hawk",
    "Dragonfly",
    "Ocelot",
    "Uakari",
    "Cuckoo",
    "Aardvark",
    "Vaquita",
    "Alpaca",
    "Nudibranch",
    "Weta",
    "Bumblebee",
    "Fairywren",
    "Emperor",
    "Bittern",
    "Puffleg",
    "Dotterel",
    "Caracara",
    "Pangasius",
    "Euploea",
    "Wolverine",
    "Buzzard",
    "Nightingale",
    "Oryx",
    "Earwig",
    "Toucan",
    "Motmot",
    "Discus",
    "Parrot",
    "Axolotl",
    "Kittiwake",
    "Leipoa",
    "Ibycter",
    "Gerbil",
    "Javan",
    "Mandrill",
    "Dunlin",
    "Hedgehog",
    "Lorikeet",
    "Gourami",
    "Okapi",
    "Clownfish",
    "Kookaburra",
    "Eryops",
    "Euphlyctis",
    "Iguana",
    "Hummingbird",
    "Eubalaena",
    "Cicada",
    "Quagga",
    "Guppy",
    "Clingfish",
    "Marmoset",
    "Bandicoot",
]
CHEF_NAME, CHARACTERS = CHARACTERS[0], CHARACTERS[1:]
